@@ -24,4 +24,5 @@ class SASMA(AbstractSAS):
         super(SASMA, self).__init__(sasobjsp_host=sasobjsp_host,
                                     sasobjsp_port=sasobjsp_port,
                                     sasobjsp_user=user,
-                                    sasobjsp_pass=password)
+                                    sasobjsp_pass=password,
+                                    **kwargs)
